﻿local Addon, Ns = ...
local GetNitroboosted = Ns.GetNitroboosted
local L = Ns.L

local EventFrame = CreateFrame("Frame", nil)
EventFrame:RegisterEvent("ADDON_LOADED")

local DefaultSettings = {
	EnableAddon = true,
	Solo = true,
	Party = true,
	Raid = true,
	PartyInstance = true,
	RaidInstance = true,
	Battleground = true
}

function GetNitroboosted:CopySettings(src, dst)
	if type(src) ~= "table" then
		return { }
	end
	if type(dst) then
		dst = { }
	end
	for k, v in pairs(src) do
		if type(v) == "table" then
			dst[k] = GetNitroboosted:CopySettings(v, dst[k])
		elseif type(v) ~= type(dst[k]) then
			dst[k] = v
		end
	end
	return dst
end

GetNitroboostedVars = GetNitroboosted:CopySettings(DefaultSettings, GetNitroboostedVars)

-- WoW version
local _, _, _, WowBuildInfo = GetBuildInfo()

-- Message Format
local MessageFormat = "%s %s!"

-- Channels
local BattlegroundChannel		= "INSTANCE_CHAT"
local RaidInstanceChannel		= "INSTANCE_CHAT"
local PartyInstanceChannel		= "INSTANCE_CHAT"
local RaidChannel				= "RAID"
local PartyChannel				= "PARTY"
local SoloChannel				= "SAY"


function GetNitroboosted:OnLoad()
	GetNitroboosted:RegisterEvents()
	SlashCmdList["GetNitroboosted"] = GetNitroboosted.SlashCommands
	SLASH_GetNitroboosted1 = "/gn"
	SLASH_GetNitroboosted2 = "/getnitroboosted"
end

function GetNitroboosted:RegisterEvents()
	EventFrame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	EventFrame:UnregisterEvent("ADDON_LOADED")
end

function GetNitroboosted:UnregisterEvents()
	EventFrame:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	EventFrame:UnregisterEvent("ADDON_LOADED")
end

function GetNitroboosted:OnEvent(event, ...)
	if event == "ADDON_LOADED" then
		local LoadedAddon = ...
		if LoadedAddon == Addon then
			if GetNitroboostedVars.EnableAddon == true then
				GetNitroboosted:OnLoad()
			end
		end
	elseif event == "COMBAT_LOG_EVENT_UNFILTERED" then
		local eventType, destGUID, id
		if WowBuildInfo ~= nil then
			if WowBuildInfo >= 50100 then
				BattlegroundChannel = "INSTANCE_CHAT"
				RaidInstanceChannel = "INSTANCE_CHAT"
				PartyInstanceChannel = "INSTANCE_CHAT"
				_, eventType, _, _, _, _, _, destGUID, _, _, _, id = ...
			elseif WowBuildInfo >= 40200 then
				BattlegroundChannel = "BATTLEGROUND"
				RaidInstanceChannel = "RAID"
				PartyInstanceChannel = "PARTY"
				_, eventType, _, _, _, _, _, destGUID, _, _, _, id = ...
			elseif WowBuildInfo >= 40100 then
				BattlegroundChannel = "BATTLEGROUND"
				RaidInstanceChannel = "RAID"
				PartyInstanceChannel = "PARTY"
				_, eventType, _, _, _, _, destGUID, _, _, id = ...
			else
				BattlegroundChannel = "BATTLEGROUND"
				RaidInstanceChannel = "RAID"
				PartyInstanceChannel = "PARTY"
				_, eventType, _, _, _, destGUID, _, _, id = ...
			end
		else
			BattlegroundChannel = "INSTANCE_CHAT"
			RaidInstanceChannel = "INSTANCE_CHAT"
			PartyInstanceChannel = "INSTANCE_CHAT"
			_, eventType, _, _, _, _, _, destGUID, _, _, _, id = ...
		end
		local inInstance, instanceType = IsInInstance()
		if IsInGroup(LE_PARTY_CATEGORY_INSTANCE) and inInstance then
			if instanceType == "pvp" then
				if GetNitroboostedVars.Battleground == true then
					if eventType == "SPELL_AURA_APPLIED" then
						if id == 94794 and destGUID == UnitGUID("player") then
							local Message = format(MessageFormat, GetSpellLink(id), L["on me"])
							SendChatMessage(Message, BattlegroundChannel)
						end
					end
				end
			elseif instanceType == "raid" then
				if GetNitroboostedVars.RaidInstance == true then
					if eventType == "SPELL_AURA_APPLIED" then
						if id == 94794 and destGUID == UnitGUID("player") then
							local Message = format(MessageFormat, GetSpellLink(id), L["on me"])
							SendChatMessage(Message, RaidInstanceChannel)
						end
					end
				end
			elseif instanceType == "party" then
				if GetNitroboostedVars.PartyInstance == true then
					if eventType == "SPELL_AURA_APPLIED" then
						if id == 94794 and destGUID == UnitGUID("player") then
							local Message = format(MessageFormat, GetSpellLink(id), L["on me"])
							SendChatMessage(Message, PartyInstanceChannel)
						end
					end
				end
			elseif instanceType == nil then
				if GetNitroboostedVars.PartyInstance == true then
					if eventType == "SPELL_AURA_APPLIED" then
						if id == 94794 and destGUID == UnitGUID("player") then
							local Message = format(MessageFormat, GetSpellLink(id), L["on me"])
							SendChatMessage(Message, PartyInstanceChannel)
						end
					end
				end
			end
		else
			if IsInRaid() == true then
				if GetNitroboostedVars.Raid == true then
					if eventType == "SPELL_AURA_APPLIED" then
						if id == 94794 and destGUID == UnitGUID("player") then
							local Message = format(MessageFormat, GetSpellLink(id), L["on me"])
							SendChatMessage(Message, RaidChannel)
						end
					end
				end
			elseif GetNumSubgroupMembers() ~= nil and GetNumSubgroupMembers() > 0 then
				if GetNitroboostedVars.Party == true then
					if eventType == "SPELL_AURA_APPLIED" then
						if id == 94794 and destGUID == UnitGUID("player") then
							local Message = format(MessageFormat, GetSpellLink(id), L["on me"])
							SendChatMessage(Message, PartyChannel)
						end
					end
				end
			elseif instanceType == "none" then
				if GetNitroboostedVars.Solo == true then
					if eventType == "SPELL_AURA_APPLIED" then
						if id == 94794 and destGUID == UnitGUID("player") then
							local Message = format(MessageFormat, GetSpellLink(id), L["on me"])
							SendChatMessage(Message, SoloChannel)
						end
					end
				end
			end
		end
	end
end

if GetNitroboostedVars.EnableAddon == true then
	EventFrame:SetScript("OnEvent", GetNitroboosted.OnEvent)
end

function GetNitroboosted.SlashCommands(command)
	local cmd = string.lower(command)
	if cmd == "" then
		GetNitroboosted:PrintSlashMenu()
	elseif cmd == "party" then
		GetNitroboostedVars.Party = not GetNitroboostedVars.Party
		if GetNitroboostedVars.Party == true then
			print("|cFF50C0FF"..L["Show in Party"]..": |cFFADFF2F"..L["[Enable]"].."|r")
		else
			print("|cFF50C0FF"..L["Show in Party"]..": |cFFFF4500"..L["[Disable]"].."|r")
		end
	elseif cmd == "raid" then
		GetNitroboostedVars.Raid = not GetNitroboostedVars.Raid
		if GetNitroboostedVars.Raid == true then
			print("|cFF50C0FF"..L["Show in Raid"]..": |cFFADFF2F"..L["[Enable]"].."|r")
		else
			print("|cFF50C0FF"..L["Show in Raid"]..": |cFFFF4500"..L["[Disable]"].."|r")
		end
	elseif cmd == "partyinstance" or cmd == "party instance" or cmd == "partyi" then
		GetNitroboostedVars.PartyInstance = not GetNitroboostedVars.PartyInstance
		if GetNitroboostedVars.PartyInstance == true then
			print("|cFF50C0FF"..L["Show in Party Instance"]..": |cFFADFF2F"..L["[Enable]"].."|r")
		else
			print("|cFF50C0FF"..L["Show in Party Instance"]..": |cFFFF4500"..L["[Disable]"].."|r")
		end
	elseif cmd == "raidinstance" or cmd == "raid instance" or cmd == "raidi" then
		GetNitroboostedVars.RaidInstance = not GetNitroboostedVars.RaidInstance
		if GetNitroboostedVars.RaidInstance == true then
			print("|cFF50C0FF"..L["Show in Raid Instance"]..": |cFFADFF2F"..L["[Enable]"].."|r")
		else
			print("|cFF50C0FF"..L["Show in Raid Instance"]..": |cFFFF4500"..L["[Disable]"].."|r")
		end
	elseif cmd == "battleground" or cmd == "bg" then
		GetNitroboostedVars.Battleground = not GetNitroboostedVars.Battleground
		if GetNitroboostedVars.Battleground == true then
			print("|cFF50C0FF"..L["Show in Battleground"]..": |cFFADFF2F"..L["[Enable]"].."|r")
		else
			print("|cFF50C0FF"..L["Show in Battleground"]..": |cFFFF4500"..L["[Disable]"].."|r")
		end
	elseif cmd == "solo" then
		GetNitroboostedVars.Solo = not GetNitroboostedVars.Solo
		if GetNitroboostedVars.Solo == true then
			print("|cFF50C0FF"..L["Show while Solo"]..": |cFFADFF2F"..L["[Enable]"].."|r")
		else
			print("|cFF50C0FF"..L["Show while Solo"]..": |cFFFF4500"..L["[Disable]"].."|r")
		end
	else
		GetNitroboosted:PrintSlashHelp()
	end
end

function GetNitroboosted:PrintSlashHelp()
	print("|cFFFFA500GetNitroboosted "..GetAddOnMetadata("GetNitroboosted", "Version").." "..L["Command List"].."|r")
	print("|cFF50C0FF/gn - |cFFFFFFA0"..L["Show Status"].."|r")
	print("|cFF50C0FF/gn solo - |cFFFFFFA0"..L["Enables or Disables announcement while Solo"].."|r")
	print("|cFF50C0FF/gn party - |cFFFFFFA0"..L["Enables or Disables announcement in Party"].."|r")
	print("|cFF50C0FF/gn raid - |cFFFFFFA0"..L["Enables or Disables announcement in Raid"].."|r")
	print("|cFF50C0FF/gn partyi - |cFFFFFFA0"..L["Enables or Disables announcement in Party Instance"].."|r")
	print("|cFF50C0FF/gn raidi - |cFFFFFFA0"..L["Enables or Disables announcement in Raid Instance"].."|r")
	print("|cFF50C0FF/gn bg - |cFFFFFFA0"..L["Enables or Disables announcement in Battleground"].."|r")
end

function GetNitroboosted:PrintSlashMenu()
	print("|cFFFFA500GetNitroboosted "..GetAddOnMetadata("GetNitroboosted", "Version").." "..L["Enabled. Type '/gn help' for options."].."|r")
	if GetNitroboostedVars.Solo == true then
		print("|cFF50C0FF"..L["Show while Solo"]..": |cFFADFF2F"..L["[Enable]"].."|r")
	else
		print("|cFF50C0FF"..L["Show while Solo"]..": |cFFFF4500"..L["[Disable]"].."|r")
	end
	if GetNitroboostedVars.Party == true then
		print("|cFF50C0FF"..L["Show in Party"]..": |cFFADFF2F"..L["[Enable]"].."|r")
	else
		print("|cFF50C0FF"..L["Show in Party"]..": |cFFFF4500"..L["[Disable]"].."|r")
	end
	if GetNitroboostedVars.Raid == true then
		print("|cFF50C0FF"..L["Show in Raid"]..": |cFFADFF2F"..L["[Enable]"].."|r")
	else
		print("|cFF50C0FF"..L["Show in Raid"]..": |cFFFF4500"..L["[Disable]"].."|r")
	end
	if GetNitroboostedVars.PartyInstance == true then
		print("|cFF50C0FF"..L["Show in Party Instance"]..": |cFFADFF2F"..L["[Enable]"].."|r")
	else
		print("|cFF50C0FF"..L["Show in Party Instance"]..": |cFFFF4500"..L["[Disable]"].."|r")
	end
	if GetNitroboostedVars.RaidInstance == true then
		print("|cFF50C0FF"..L["Show in Raid Instance"]..": |cFFADFF2F"..L["[Enable]"].."|r")
	else
		print("|cFF50C0FF"..L["Show in Raid Instance"]..": |cFFFF4500"..L["[Disable]"].."|r")
	end
	if GetNitroboostedVars.Battleground == true then
		print("|cFF50C0FF"..L["Show in Battleground"]..": |cFFADFF2F"..L["[Enable]"].."|r")
	else
		print("|cFF50C0FF"..L["Show in Battleground"]..": |cFFFF4500"..L["[Disable]"].."|r")
	end
end